local MemoryVersion = 1;
local SettingsVersion = 1;

GlobalShortTermMemory = {
	__VERSION__ = MemoryVersion,
	RealID = {}
};
ShortTermMemory = {
	__VERSION__ = MemoryVersion;
	Default = {}
};
ShortTermMemorySettings = {
	SettingsVersion = SettingsVersion,
	MemorySets = {
		Default = {
			Scope = "CHARACTER",
			Length = 100,
			Events = {
				SYSTEM = false,        BATTLEGROUND_LEADER = true,
				SAY = true,            GUILD = true,
				EMOTE = true,          GUILD_OFFICER = true,   
				YELL = true,           MONSTER_SAY = true,  
				WHISPER = true,        MONSTER_YELL = true,  
				PARTY = true,          MONSTER_EMOTE = true, 
				RAID = true,           MONSTER_WHISPER = true,  
				RAID_LEADER = true,    MONSTER_BOSS_EMOTE = true,    
				RAID_WARNING = true,   MONSTER_BOSS_WHISPER = true,      
				BATTLEGROUND = true,   ERRORS = false,     
				AFK = true,            SKILL = true,
				DND = true,            LOOT = true,
				IGNORED = true,        MONEY = true,
				BG_HORDE = true,       OPENING = false,
				BG_ALLIANCE = true,    TRADESKILLS = false,
				BG_NEUTRAL = true,     PET_INFO = false,
				COMBAT_XP_GAIN = false,COMBAT_MISC_INFO = false,
				ACHIEVEMENT = true,    COMBAT_HONOR_GAIN = false,
				CHANNEL = true,        COMBAT_FACTION_CHANGE = false,
				GUILD_ACHIEVEMENT = true,
				BN_WHISPER = true,     BN_CONVERSATION = true,
				BN_TOAST = false,
			},
		},
		RealID = {
			Scope = "GLOBAL",
			Length = 100,
			Events = {
				BN_WHISPER = true,
				BN_CONVERSATION = true,
			},
		},
		--- TODO: Scopes: Guild, ServerFaction, Server
	},
};

local Events = {
	SYSTEM = true,        BATTLEGROUND_LEADER = true,
	SAY = true,            GUILD = true,
	EMOTE = true,          GUILD_OFFICER = true,   
	YELL = true,           MONSTER_SAY = true,  
	WHISPER = true,        MONSTER_YELL = true,  
	PARTY = true,          MONSTER_EMOTE = true, 
	RAID = true,           MONSTER_WHISPER = true,  
	RAID_LEADER = true,    MONSTER_BOSS_EMOTE = true,    
	RAID_WARNING = true,   MONSTER_BOSS_WHISPER = true,      
	BATTLEGROUND = true,   ERRORS = true,     
	AFK = true,            SKILL = true,
	DND = true,            LOOT = true,
	IGNORED = true,        MONEY = true,
	BG_HORDE = true,       OPENING = true,
	BG_ALLIANCE = true,    TRADESKILLS = true,
	BG_NEUTRAL = true,     PET_INFO = true,
	COMBAT_XP_GAIN = true, COMBAT_MISC_INFO = true,
	ACHIEVEMENT = true,    COMBAT_HONOR_GAIN = true,
	CHANNEL = true,        COMBAT_FACTION_CHANGE = true,
	GUILD_ACHIEVEMENT = true,
	BN_WHISPER = true,     BN_CONVERSATION = true,
	BN_TOAST = true,
};


--[[ OPTIONS ]]--
local L = LibStub("AceLocale-3.0"):GetLocale("Short term");
local LG = LibStub("AceLocale-3.0"):GetLocale("Short term message groups");

local optionstable = {
	type = "group",
	name = L.SHORT_TERM_MEMORY,
	childGroups = "tab",
	args = {
		com = {
			name = L.DISCLAIMER,
			type = "description",
			order = 2,
		},
		gui = {
			name = "Config dialog",
			order = 1,
			guiHidden = true,
			dialogHidden = true,
			type = "execute",
			func = function() InterfaceOptionsFrame_OpenToCategory(optionstablecategory); end,
		},
		general = {
			name = L.GENERAL,
			order = 3,
			type = "group",
			args = {
				addgroupinput = {
					name = L.ADDGROUP,
					desc = L.ADDGROUP_DESC,
					type = "input",
					set = function(info,val)
						val = tonumber(val);
						ShortTermMemorySettings.BacklogLength = val;
						-- lets just hope the player does not do this too often.
						ShortTerm.cache = LibRingCache:createBlackbox(val, ShortTermMemory);
					end,
					get = function() return ""; end,
				},
				addgroup = {
					name = L.ADDGROUP,
					desc = L.ADDGROUP_DESC,
					type = "execute",
					func = function() return; end,
				},
				--[[
				backlogtime = {
					name = L.BACKLOG_TIME,
					desc = L.BACKLOG_TIME_DESC,
					type = "range",
					order = 2,
					disabled = true,
					min = 0,
					max = 72,
					step = 1,
					bigStep = 1,
					width = "full",
					set = function(info,val)
						ShortTermMemorySettings.BacklogTimeOut = val;
					end,
					get = function() return ShortTermMemorySettings.BacklogTimeOut or 2; end,
				},
				showfragments = {
					name = L.SHOW_FRAGMENTS,
					desc = L.SHOW_FRAGMENTS_DESC,
					type = "toggle",
					order = 3,
					set = function(info,val)
						ShortTermMemorySettings.ShowFragments = val;
					end,
					get = function()
						return ShortTermMemorySettings.ShowFragments or false;
					end,
				},
				]]
			},
		},
		-- more groups go here
	},
};

local MessageGroup = {
	name = L.TYPES,
	desc = L.TYPES_DESC,
	order = 3,
	type = "group",
	args = {
		desc = {
			name = L.TYPES_DESC,
			type = "description",
			order = 1,
		},
		backloglength = {
			name = L.BACKLOG_LENGTH,
			desc = L.BACKLOG_LENGTH_DESC,
			type = "range",
			order = 1,
			min = 10,
			max = 500,
			step = 1,
			bigStep = 10,
			width = "full",
			set = function(info,val)
				val = tonumber(val);
				ShortTermMemorySettings.BacklogLength = val;
				-- for now lets just hope the player does not do this too often.
				ShortTerm.cache = LibRingCache:createBlackbox(val, ShortTermMemory);
			end,
			get = function() return ShortTermMemorySettings.BacklogLength or 50; end,
		},
		types = {
			name = L.TYPES2,
			type = "multiselect",
			width = "full",
			values = function()
				local ks = {};
				for k,v in pairs(events) do
					ks[k] = LG[k];
				end
				return ks;
			end,
			get = function(info, index)
				local parent = info[#info-1];
				_G.INSPECTINFO = info;
				return ShortTermMemorySettings.MemorySets["Default"].Events[index] or false;
			end,
			set = function(info, index, val)
				ShortTermMemorySettings.Events[index] = val or false;
			end,
		}
	},
};


local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

local function shallowcopy(tbl)
	local ret = {};
	for k, v in pairs(tbl) do
		ret[k] = v;
	end
	return ret;
end

AceConfig:RegisterOptionsTable("Short term memory", function()
	for k, v in pairs(ShortTermMemorySettings.MemorySets) do
		local cpy = shallowcopy(MessageGroup);
		cpy.name = L[k];
		--cpy.data = 
		optionstable.args[k] = shallowcopy(MessageGroup);
	end
	return optionstable;
end);
optionstablecategory = AceConfigDialog:AddToBlizOptions("Short term memory", nil, nil);